#ifndef _POWERRPC_H
#define _POWERRPC_H

#ifdef __cplusplus
extern "C" {
#endif

typedef void (*local_call_t) (void*, void*);
typedef void (*free_call_t) (void*);
typedef void (*before_fork_call_t) (void);
typedef void (*pwrpc_perr_call_t) (char*);


enum {POWERRPC_FORKING=1, POWERRPC_THREADING=2};

typedef struct {
    caddr_t         pover;
    int             over_size;
    caddr_t         pback;
    int             back_size;
    SVCXPRT        *ptransport;
    struct          svc_req *psvc_req;
    void            (*proc)(SVCXPRT*);
    local_call_t    local;
    free_call_t     free_after;
    xdrproc_t       xdr_over;
    xdrproc_t       xdr_back;
    int             reply_now;
    int             mt_on_call;
}        pwrpc_call_struct;

void pwrpc_report_event(char*);

void pw_serv_mainloop(SVCXPRT*tcp_master, SVCXPRT* udp_master, int dofork, int exit_idle);

void pw_serv_once(struct timeval *timeout);

void pw_serv_fork(fd_set*, SVCXPRT*, SVCXPRT*);

void pw_fork_on_call(int*);

void pw_serv_init(void);

void pw_serv_validate(void);

CLIENT * pw_clnt_create(const char*host, short port, u_long prog, u_long vers, char* proto, u_int sendsz, u_int recvsz);

int pw_serv_fd(u_short port, const char* type);

void pw_serv_input_handler(int fd, void (*handler)(int fd));

void pw_serv_output_handler(int fd, void (*handler)(int fd));

void pw_serv_exception_handler(int fd, void (*handler)(int fd));

void pw_serv_call(pwrpc_call_struct * pcall);

void pw_set_err_report_func(pwrpc_perr_call_t err_call);

int pw_serv_async_enable() ;

int pw_serv_async_cancel() ;

int pw_serialize(FILE* fp, xdrproc_t , void * pdata, enum xdr_op op);

void pw_free_reference(xdrproc_t, void*, u_int size);

enum clnt_stat pw_clnt_errno(CLIENT *cl);

#ifdef __cplusplus
};
#endif

#endif
